forgeschema -s c:/dev/work/forge/tcli/103120/schema/xsd/ts_103120_Core.xsd ^
-u c:/dev/work/forge/tcli/103120/schema/xsd/ ^
-u c:/dev/work/forge/tcli/103120/dictionaries ^
-u c:/dev/work/forge/tcli/103280 ^
-i stdin ^